#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import shutil
from django.conf import settings
from bigbuild.views import PageArchiveView
from bigbuild.models import PageList, Page
from django.core.management import call_command
from django.core.management.base import BaseCommand, CommandError


class Command(BaseCommand):
    help = "Archive a page directory by permanently rendering its HTML"
    args = "<slug>"

    def add_arguments(self, parser):
        parser.add_argument('slug', nargs='+', type=str)
        parser.add_argument(
            '--keep-page',
            action='store_true',
            dest='keep_page',
            default=False,
            help='Do not delete the page directory as part of the archival'
        )

    def get_build_directory(self):
        """
        Returns the location of django-bakery's build directory.
        """
        if getattr(settings, 'BUILD_DIR', False):
            return settings.BUILD_DIR
        else:
            return os.path.join(settings.BASE_DIR, '.build')

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        # Loop through the slugs
        for slug in options['slug']:
            # Pull the object
            try:
                p = PageList()[slug]
            except:
                raise CommandError("Slug provided (%s) does not exist" % slug)

            if not isinstance(p, Page):
                raise CommandError("Slug (%s) is not a Page object" % slug)

            # Build it
            PageArchiveView().build_object(p)

            # If the retired directory exists, kill it
            if os.path.exists(p.archive_static_directory_path):
                shutil.rmtree(p.archive_static_directory_path)  # pragma: no cover

            # Save that directory to the retired folder
            shutil.copytree(
                os.path.join(self.get_build_directory(), p.get_absolute_url()[1:]),
                p.archive_static_directory_path,
            )

            # Save the metadata to the retired folder
            frontmatter_path = os.path.join(p.archive_static_directory_path, 'metadata.md')
            p.write_frontmatter(frontmatter_path)

            # Copy the dynamic page folder to its archival location
            shutil.copytree(p.dynamic_directory_path, p.archive_dynamic_directory_path)

            # Delete the page folder
            if not options['keep_page']:
                shutil.rmtree(p.dynamic_directory_path)

        # Update the cache
        call_command("cachepages")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
from bigbuild.models import BasePage


class ArchivedPage(BasePage):
    """
    An archived custom page.
    """
    @property
    def frontmatter_path(self):
        """
        Returns the metadata.md path where this page will be configured.
        """
        return os.path.join(self.archive_static_directory_path, 'metadata.md')



import os
from setuptools import setup
from distutils.core import Command


class TestCommand(Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        from django.conf import settings
        settings.configure(
            DATABASES={
                'default': {
                    'NAME': 'test.db',
                    'TEST_NAME': 'test.db',
                    'ENGINE': 'django.db.backends.sqlite3'
                }
            },
            INSTALLED_APPS = (
                'django.contrib.auth',
                'django.contrib.contenttypes',
                'django.contrib.sessions',
                'django.contrib.staticfiles',
                'compressor',
                'greeking',
                'bigbuild',
            ),
            TEMPLATES=[
                {
                    'BACKEND': 'django.template.backends.django.DjangoTemplates',
                    'DIRS': [],
                    'APP_DIRS': False,
                    'OPTIONS': {},
                },
            ],
            BASE_DIR=os.path.dirname(__file__),
            BAKERY_GZIP=False,
            BAKERY_VIEWS=(
                'bigbuild.views.IndexRedirectView',
                'bigbuild.views.PageListView',
                'bigbuild.views.PageDetailView',
                'bigbuild.views.Static404View',
                'bigbuild.views.RobotsView',
                'bigbuild.sitemaps.SitemapView',
                'bigbuild.sitemaps.GoogleNewsSitemapView',
                'bigbuild.feeds.LatestPages',
            ),
            STATIC_ROOT="./.static",
            STATIC_URL = "/static/",
            COMPRESS_ENABLED = True,
            COMPRESS_JS_COMPRESSOR = "bigbuild.compressors.SimpleJsCompressor",
            COMPRESS_CSS_COMPRESSOR = "bigbuild.compressors.SimpleCssCompressor",
            STATICFILES_FINDERS = (
                'compressor.finders.CompressorFinder',
            ),
            COMPRESS_CSS_FILTERS = [
                'compressor.filters.cssmin.rCSSMinFilter'
            ],
            COMPRESS_JS_FILTERS = [
                'compressor.filters.jsmin.JSMinFilter',
            ],
            COMPRESS_PRECOMPILERS = (
                ('text/coffeescript', 'coffee --compile --stdio'),
                ('text/less', 'lessc {infile} {outfile}'),
                ('text/x-sass', 'sass {infile} {outfile}'),
            ),
            COMPRESS_CACHEABLE_PRECOMPILERS = (),
            USE_TZ = True,
            ROOT_URLCONF = 'bigbuild.urls',
            BIGBUILD_GIT_BRANCH = os.environ.get("BIGBUILD_GIT_BRANCH", None),
            BIGBUILD_BRANCH_BUILD = True,
        )
        from django.core.management import call_command
        import django
        django.setup()
        call_command('test', 'bigbuild')


setup(
    name='django-bigbuild',
    version='0.3.8',
    description='The open-source engine that powers bigbuilder, the Los Angeles Times Data Desk\'s system for publishing standalone pages',
    author='The Los Angeles Times Data Desk',
    author_email='datadesk@latimes.com',
    url='http://www.github.com/datadesk/django-bigbuild/',
    zip_safe=False,
    packages=(
        'bigbuild',
        'bigbuild.management',
        'bigbuild.management.commands',
        'bigbuild.models',
        'bigbuild.templatetags',
    ),
    include_package_data=True,
    license="MIT",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Framework :: Django',
        'Framework :: Django :: 1.10',
        'License :: OSI Approved :: MIT License',
    ],
    install_requires=[
        'python-frontmatter>=0.2.1',
        'django-bakery>=0.8.11',
        'validictory>=1.0.1',
        'django-compressor>=2.0',
        'greeking>=2.1.3',
        'GitPython>=2.0.8',
        'python-dateutil>=2.6.0',
        'pytz',
        'six',
    ],
    cmdclass={'test': TestCommand}
)



import os
from git import Repo
from django.conf import settings
default_app_config = 'bigbuild.apps.BigbuildConfig'


def get_page_directory():
    """
    Returns the PAGE_DIR where dynamic pages are configured.
    """
    # Return the PAGE_DIR settings, if it's been set, or fall back to the default.
    path = getattr(
        settings,
        'BIGBUILD_PAGE_DIR',
        os.path.join(settings.BASE_DIR, 'pages')
    )
    # Make the directory if it doesn't exist already
    os.path.exists(path) or os.mkdir(path)
    # Return the path
    return path


def get_archive_directory():
    """
    Returns the BIGBUILD_ARCHIVE_DIR where archived pages are configured.
    """
    # Return the BIGBUILD_ARCHIVE_DIR settings, if it's been set, or fall back to the default.
    path = getattr(
        settings,
        'BIGBUILD_ARCHIVE_DIR',
        os.path.join(settings.BASE_DIR, '.archive')
    )

    # Make the directory if it doesn't exist already
    os.path.exists(path) or os.mkdir(path)

    # Make the two expected subdirectories if they do not already exist
    static_archive = os.path.join(path, 'static')
    dynamic_archive = os.path.join(path, 'dynamic')
    os.path.exists(static_archive) or os.mkdir(static_archive)
    os.path.exists(dynamic_archive) or os.mkdir(dynamic_archive)

    # Return the roo archive path
    return path


def get_repo_branch():
    """
    Returns the name of the current git branch.
    """
    if getattr(settings, 'BIGBUILD_GIT_BRANCH', None):
        return settings.BIGBUILD_GIT_BRANCH
    repo_path = getattr(settings, 'BIGBUILD_GIT_DIR', settings.BASE_DIR)
    branch = Repo(repo_path).active_branch
    return branch.name


def get_base_url():
    """
    Returns the base URL for site.
    """
    base_url = getattr(settings, 'BIGBUILD_BASE_URL', '')
    if base_url.startswith("/"):
        base_url = base_url[1:]

    if getattr(settings, 'BIGBUILD_BRANCH_BUILD', False):
        # Get the branch name
        repo_branch = get_repo_branch()
        # Put the branch name ahead of the base url
        return os.path.join("/", repo_branch, base_url)
    else:
        return os.path.join("/", base_url)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import shutil
from django.urls import reverse
from django.http import Http404
from django.conf import settings
from django.views.static import serve
from bigbuild.models import PageList, Page, ArchivedPage
from bigbuild import get_page_directory, get_archive_directory, get_base_url
from bigbuild.management.commands.build import Command as Build
from bakery.views import (
    BuildableTemplateView,
    BuildableDetailView,
    Buildable404View,
    BuildableRedirectView
)


class BigBuildMixin(object):
    """
    A class-based view mixin with utilities for our bigbuild pages.
    """
    def joinpath(self, *args):
        """
        Returns a joined path with the front slash of the first argument always sliced off.

        Allows for automatically generated URLs from elsewhere in the app be used as relative build paths.
        """
        args = list(args)
        first = args.pop(0)[1:]
        args.insert(0, first)
        return os.path.join(*args)


class Static404View(Buildable404View):
    """
    The 404 page.
    """
    template_name = "bigbuild/404.html"
    build_path = "404.html"


class RobotsView(BuildableTemplateView, BigBuildMixin):
    """
    The robots.txt file.
    """
    template_name = "bigbuild/robots.txt"

    @property
    def build_path(self):
        return self.joinpath(get_base_url(), "robots.txt")


class IndexRedirectView(BuildableRedirectView, BigBuildMixin):
    """
    Redirects the root URL to /projects/
    """
    url = get_base_url()

    @property
    def build_path(self):
        return self.joinpath(reverse('bigbuild-index-redirect'), "index.html")


class PageListView(BuildableTemplateView, BigBuildMixin):
    """
    Renders a homepage with a list of all the pages.
    """
    template_name = "bigbuild/page_list.html"

    @property
    def build_path(self):
        return self.joinpath(reverse('bigbuild-page-list'), "index.html")

    def get_context_data(self):
        PAGE_PUBLICATION_STATUS = getattr(
            settings,
            'PAGE_PUBLICATION_STATUS',
            'working'
        )
        return {
            'object_list': [p for p in PageList() if p.show_in_feeds],
            'PAGE_PUBLICATION_STATUS': PAGE_PUBLICATION_STATUS
        }


class PageDetailView(BuildableDetailView):
    """
    Renders one of the page objects as an HTML response.
    """
    _queryset = None

    def get_queryset(self):
        if self._queryset:
            return self._queryset
        else:
            qs = PageList()
            self._queryset = qs
            return qs

    def get_object(self):
        qs = self.get_queryset()
        return qs[self.kwargs.get("slug")]

    def get_template_names(self):
        return [os.path.join(self.object.slug, "index.html")]

    def get_context_data(self, object=None):
        return {
            'metadata': self.object.metadata,
            'object': self.object,
            'STATIC_URL': self.object.get_static_url()
        }

    def get_build_directory(self):
        if getattr(settings, 'BUILD_DIR', False):
            return settings.BUILD_DIR
        else:
            return os.path.join(settings.BASE_DIR, '.build')

    def build_static_directory(self, obj):
        """
        Builds an object's static subdirectory.
        """
        source_dir = obj.static_path
        target_dir = os.path.join(
            self.get_build_directory(),
            obj.get_static_url()[1:]
        )
        if settings.BAKERY_GZIP:
            Build().copytree_and_gzip(
                source_dir,
                target_dir
            )
        else:
            os.path.exists(target_dir) and shutil.rmtree(target_dir)
            shutil.copytree(source_dir, target_dir)

    def build_object(self, obj):
        if isinstance(obj, Page):
            super(PageDetailView, self).build_object(obj)
            self.build_static_directory(obj)
        elif isinstance(obj, ArchivedPage):
            target = os.path.join(self.get_build_directory(), obj.get_absolute_url()[1:])
            os.path.exists(target) and shutil.rmtree(target)
            if settings.BAKERY_GZIP:
                Build().copytree_and_gzip(
                    obj.archive_static_directory_path,
                    target
                )
            else:
                shutil.copytree(obj.archive_static_directory_path, target)

    def build_queryset(self):
        [self.build_object(o) for o in self.get_queryset()]


class PageArchiveView(PageDetailView):

    def get_context_data(self, object=None):
        context = super(PageArchiveView, self).get_context_data(object=object)
        context['ARCHIVAL'] = True
        return context


def page_static_serve(request, slug, path):  # pragma: no cover
    """
    Serves files from the ./static/ subdirectory for every page.
    """
    # Set the path to the file
    path = os.path.join(slug, 'static', path)
    # First test if it can be found in the PAGE_DIR
    try:
        return serve(
            request,
            path,
            document_root=get_page_directory(),
            show_indexes=True
        )
    # If it can't try to fallback to the BIGBUILD_ARCHIVE_DIR
    except Http404:
        return serve(
            request,
            path,
            document_root=os.path.join(get_archive_directory(), 'static'),
            show_indexes=True
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.urls import reverse
from bigbuild.models import PageList
from bakery.views import BuildableListView


class SitemapView(BuildableListView):
    """
    A list of all static pages in a Sitemap ready for Google.
    """
    template_name = 'bigbuild/sitemap.xml'

    @property
    def build_path(self):
        return reverse('bigbuild-sitemap')[1:]

    def get_queryset(self):
        return [p for p in PageList() if p.show_in_feeds]

    def render_to_response(self, context):
        return super(SitemapView, self).render_to_response(
            context,
            content_type='text/xml'
        )


class GoogleNewsSitemapView(SitemapView):
    """
    A list of the most recent graphics in a Sitemap ready for Google News.
    """
    template_name = 'bigbuild/google-news-sitemap.xml'

    @property
    def build_path(self):
        return reverse('bigbuild-google-news-sitemap')[1:]

    def get_queryset(self):
        return [p for p in PageList() if p.show_in_feeds][:25]
